import { appLogger } from '#application/logger'
import { BaseRepository } from '#application/base/baseRepository'
import { CharacterHair } from '#entities/characterHair'

class CharacterHairRepository extends BaseRepository {
  async getFirst() {
    try {
      const repository = this.em.getRepository(CharacterHair)
      return await repository.findOne({ id: { $exists: true } }, { populate: ['*'] })
    } catch (error: any) {
      appLogger.error(`Failed to get first character hair: ${error instanceof Error ? error.message : String(error)}`)
      return null
    }
  }

  async getAll() {
    try {
      const repository = this.em.getRepository(CharacterHair)
      return await repository.findAll({ populate: ['*'] })
    } catch (error: any) {
      appLogger.error(`Failed to get all character hair: ${error instanceof Error ? error.message : String(error)}`)
      return null
    }
  }

  async getAllSelectable() {
    try {
      const repository = this.em.getRepository(CharacterHair)
      return await repository.find({ isSelectable: true }, { populate: ['*'] })
    } catch (error: any) {
      appLogger.error(`Failed to get selectable character hair: ${error instanceof Error ? error.message : String(error)}`)
      return null
    }
  }

  async getById(id: number) {
    try {
      const repository = this.em.getRepository(CharacterHair)
      return await repository.findOne({ id }, { populate: ['*'] })
    } catch (error: any) {
      appLogger.error(`Failed to get character hair by ID: ${error instanceof Error ? error.message : String(error)}`)
      return null
    }
  }
}

export default new CharacterHairRepository()
