import { appLogger } from '#application/logger'
import { BaseRepository } from '#application/base/baseRepository'
import { Chat } from '#entities/chat'

class ChatRepository extends BaseRepository {
  async getById(id: number): Promise<Chat[]> {
    try {
      const repository = this.em.getRepository(Chat)
      return await repository.find(
        {
          id
        },
        { populate: ['*'] }
      )
    } catch (error: any) {
      appLogger.error(`Failed to get chat by ID: ${error instanceof Error ? error.message : String(error)}`)
      return []
    }
  }

  async getAll(): Promise<Chat[]> {
    try {
      const repository = this.em.getRepository(Chat)
      return await repository.findAll({ populate: ['*'] })
    } catch (error: any) {
      appLogger.error(`Failed to get all chats: ${error instanceof Error ? error.message : String(error)}`)
      return []
    }
  }

  async getByCharacterId(characterId: number): Promise<Chat[]> {
    try {
      const repository = this.em.getRepository(Chat)
      return await repository.find({ character: characterId }, { populate: ['*'] })
    } catch (error: any) {
      appLogger.error(`Failed to get chats by character ID: ${error instanceof Error ? error.message : String(error)}`)
      return []
    }
  }

  async getByZoneId(zoneId: number): Promise<Chat[]> {
    try {
      const repository = this.em.getRepository(Chat)
      return await repository.find({ zone: zoneId }, { populate: ['*'] })
    } catch (error: any) {
      appLogger.error(`Failed to get chats by zone ID: ${error instanceof Error ? error.message : String(error)}`)
      return []
    }
  }
}

export default new ChatRepository()
