import { appLogger } from '#application/logger'
import { BaseRepository } from '#application/base/baseRepository'
import { User } from '#entities/user'

class UserRepository extends BaseRepository {
  async getById(id: number) {
    try {
      const repository = this.em.getRepository(User)
      return await repository.findOne({ id }, { populate: ['*'] })
    } catch (error: any) {
      appLogger.error(`Failed to get user by ID: ${error instanceof Error ? error.message : String(error)}`)
      return null
    }
  }

  async getByUsername(username: string) {
    try {
      const repository = this.em.getRepository(User)
      return await repository.findOne({ username }, { populate: ['*'] })
    } catch (error: any) {
      appLogger.error(`Failed to get user by username: ${error instanceof Error ? error.message : String(error)}`)
      return null
    }
  }

  async getByEmail(email: string) {
    try {
      const repository = this.em.getRepository(User)
      return await repository.findOne({ email }, { populate: ['*'] })
    } catch (error: any) {
      appLogger.error(`Failed to get user by email: ${error instanceof Error ? error.message : String(error)}`)
      return null
    }
  }
}

export default new UserRepository()
