import { gameLogger } from '#application/logger'
import { Server } from 'socket.io'
import { TSocket } from '#application/types'
import ChatRepository from '#repositories/chatRepository'
import { Chat } from '#entities/chat'
import ZoneRepository from '#repositories/zoneRepository'
import CharacterRepository from '#repositories/characterRepository'

class ChatService {
  async sendZoneMessage(io: Server, socket: TSocket, message: string, characterId: number, zoneId: number): Promise<boolean> {
    try {
      const character = await CharacterRepository.getById(characterId)
      if (!character) return false

      const zone = await ZoneRepository.getById(zoneId)
      if (!zone) return false

      const newChat = new Chat()

      newChat.setCharacter(character).setZone(zone).setMessage(message)

      await newChat.save()

      const chat = await ChatRepository.getById(newChat.id)
      if (!chat) return false

      io.to(zoneId.toString()).emit('chat:message', chat)
      return true
    } catch (error: any) {
      gameLogger.error(`Failed to save chat message: ${error instanceof Error ? error.message : String(error)}`)
      return false
    }
  }
}

export default ChatService
